import copy
from operator import itemgetter
from .cmmn import inplace_wrapper
from .map import mapv


@inplace_wrapper
def sort(ol,**kwargs):
    ol.sort()
    return(ol)



@inplace_wrapper
def sort_refer_to(l,referer,reverse=False,**kwargs):
    if("mode" in kwargs):
        mode = kwargs["mode"]
    else:
        mode = "both"
    tl =[]
    length = l.__len__()
    for i in range(0,length):
        ele = (l[i],referer[i])
        tl.append(ele)
    tl = sorted(tl,key=itemgetter(1),reverse=reverse)
    sorted_l =[]
    sorted_r = []
    for i in range(0,length):
        sorted_l.append(tl[i][0])
        sorted_r.append(tl[i][1])
    if(mode == "only-list"):
        return(sorted_l)
    elif(mode == "only-referer"):
        return(referer)
    else:
        return({"list":sorted_l,"referer":sorted_r})



def sort_sublist(ol,sub):
    def cond_func(ele,ol):
        index = ol.index(ele)
        return(index)
    indexes = mapv(sub,cond_func,ol)
    nsub = sort_refer_to(sub,indexes)['list']
    return(nsub)


def bat_sort(referer,*lists,**kwargs):
    if('reverse' in kwargs):
        reverse = kwargs['reverse']
    else:
        reverse = False
    length = referer.__len__()
    indexes = list(range(0,length))
    rslt = sort_refer_to(indexes,referer,reverse=reverse)
    referer = rslt['referer']
    indexes = rslt['list']
    rslt = []
    lists = copy.deepcopy(list(lists))
    for i in range(0,lists.__len__()):
        l = lists[i]
        nl = []
        for j in range(0,length):
            loc = indexes[j]
            nl.append(l[loc])
        rslt.append(nl)
    return(tuple(rslt))



def sort_dtb(dtb,*cond_keys,**kwargs):
    def default_eq_func(value1,value2):
        cond = (value1 == value2)
        return(cond)
    def default_gt_func(value1,value2):
        cond = (value1 > value2)
        return(cond)
    def default_lt_func(value1,value2):
        cond = (value1 < value2)
        return(cond)
    if('eq_func' in kwargs):
        eq_func = kwargs['eq_func']
    else:
        eq_func = default_eq_func
    if('gt_func' in kwargs):
        gt_func = kwargs['gt_func']
    else:
        gt_func = default_gt_func
    if('lt_func' in kwargs):
        lt_func = kwargs['lt_func']
    else:
        lt_func = default_lt_func
    if('reverse' in kwargs):
        reverse = kwargs['reverse']
    else:
        reverse = False
    keys = cond_keys
    def cmp_dict(d1,d2):
        '''
        '''
        length = keys.__len__()
        for i in range(0,length):
            key = keys[i]
            cond = eq_func(d1[key],d2[key])
            if(cond):
                pass
            else:
                cond = gt_func(d1[key],d2[key])
                if(cond):
                    return(1)
                else:
                    return(-1)
        return(0)
    ndl = dtb
    ndl = sorted(ndl,key=functools.cmp_to_key(cmp_dict),reverse=reverse)
    return(ndl)






def find_lst_ipair_when_fstltsnd(arr):
    lngth = len(arr)
    for snd in range(lngth-1,0,-1):
        fst = snd - 1
        if(arr[fst]<arr[snd]):
            return((fst,snd))
        else:
            pass
    return(None)

def find_lst_vpair_when_fstltsnd(arr):
    lngth = len(arr)
    for snd in range(lngth-1,0,-1):
        fst = snd - 1
        if(arr[fst]<arr[snd]):
            return((arr[fst],arr[snd]))
        else:
            pass
    return(None)


def find_lst_ipair_when_fstgtsnd(arr):
    lngth = len(arr)
    for snd in range(lngth-1,0,-1):
        fst = snd - 1
        if(arr[fst]>arr[snd]):
            return((fst,snd))
        else:
            pass
    return(None)

def find_lst_vpair_when_fstgtsnd(arr):
    lngth = len(arr)
    for snd in range(lngth-1,0,-1):
        fst = snd - 1
        if(arr[fst]>arr[snd]):
            return((arr[fst],arr[snd]))
        else:
            pass
    return(None)




def fltrfivo(ol,fltr_funcs,fltr_func_other_args_array):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        index = i
        value = ol[i]
        fltr_func = fltr_funcs[i]
        other_args = fltr_func_other_args_array[i]
        cond = fltr_func(index,value,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)    



def fltrfiv(ol,fltr_funcs,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        index = i
        value = ol[i]
        fltr_func = fltr_funcs[i]
        cond = fltr_func(index,value,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)


def fltrfio(ol,fltr_funcs,fltr_func_other_args_array):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        index = i
        fltr_func = fltr_funcs[i]
        other_args = fltr_func_other_args_array[i]
        cond = fltr_func(index,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



def fltrfvo(ol,fltr_funcs,fltr_func_other_args_array):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        value = ol[i]
        fltr_func = fltr_funcs[i]
        other_args = fltr_func_other_args_array[i]
        cond = fltr_func(value,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)


def fltrivo(ol,fltr_func,fltr_func_other_args_array):
    lngth = ol.__len__()
    fltr_func = fltr_func
    rslt = []
    for i in range(0,lngth):
        index = i
        value = ol[i]
        other_args = fltr_func_other_args_array[i]
        cond = fltr_func(index,value,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



def fltrfio(ol,fltr_funcs,fltr_func_other_args_array):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        index = i
        fltr_func = fltr_funcs[i]
        other_args = fltr_func_other_args_array[i]
        cond = fltr_func(index,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



def fltrfi(ol,fltr_funcs,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        index = i
        fltr_func = fltr_funcs[i]
        cond = fltr_func(index,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



def fltrfv(ol,fltr_funcs,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        value = ol[i]
        fltr_func = fltr_funcs[i]
        cond = fltr_func(value,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



def fltrfo(ol,fltr_funcs,fltr_func_other_args_array):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        fltr_func = fltr_funcs[i]
        other_args = fltr_func_other_args_array[i]
        cond = fltr_func(*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



def fltriv(ol,fltr_func,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        index = i
        value = ol[i]
        cond = fltr_func(index,value,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



def fltrio(ol,fltr_func,fltr_func_other_args_array):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        index = i
        other_args = fltr_func_other_args_array[i]
        cond = fltr_func(index,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)


def fltrvo(ol,fltr_func,fltr_func_other_args_array):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        value = ol[i]
        other_args = fltr_func_other_args_array[i]
        cond = fltr_func(value,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



def fltrf(ol,fltr_funcs,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        fltr_func = fltr_funcs[i]
        cond = fltr_func(*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



def fltri(ol,fltr_func,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        index = i
        cond = fltr_func(index,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)


def fltrv(ol,fltr_func,*other_args):
    lngth = ol.__len__()
    other_args = list(other_args)
    rslt = []
    for i in range(0,lngth):
        value = ol[i]
        cond = fltr_func(value,*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



def fltro(ol,fltr_func,fltr_func_other_args_array):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        other_args = fltr_func_other_args_array[i]
        cond = fltr_func(*other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)


def fltriv_with_dual(ol,fltr_func,index_fltr_func,fltr_func_other_args,index_fltr_func_other_args):
    lngth = ol.__len__()
    rslt = []
    for i in range(0,lngth):
        index = index_fltr_func(i,*index_fltr_func_other_args)
        value = fltr_func(index,ol[i],*_fltr_func_other_args)
        if(cond):
            rslt.append(ol[i])
        else:
            pass
    return(rslt)



####
def lngth_lt(ol,lngth):
    return(fltrv(ol,lambda v:len(v)<lngth))

def lngth_le(ol,lngth):
    return(fltrv(ol,lambda v:len(v)<=lngth))

def lngth_eq(ol,lngth):
    return(fltrv(ol,lambda v:len(v)==lngth))

def lngth_ge(ol,lngth):
    return(fltrv(ol,lambda v:len(v)>=lngth))

def lngth_gt(ol,lngth):
    return(fltrv(ol,lambda v:len(v)>lngth))





from .cmmn import dflt_kwargs
from .map import mapiv

def vil_dict(l):
    st = set({})
    rslt = {}
    for i in range(len(l)):
        v = l[i]
        if(v in st):
            rslt[v].append(i)
        else:
            st.add(v)
            rslt[v] = [i]
    return(rslt)


def ivdict(ol):
    d = {}
    for i in range(ol.__len__()):
        d[i] = ol[i]
    return(d)

def vidict(arr):
    d = {}
    for i in range(arr.__len__()):
        d[arr[i]] = i
    return(d)

def mirror_dict(arr):
    d = {}
    for i in range(arr.__len__()):
        d[i] = arr[i]
        d[arr[i]] = i
    return(d)


def table(l,**kwargs):
    iname = dflt_kwargs("iname","index",**kwargs)
    vname =  dflt_kwargs("vname","value",**kwargs)
    dl = mapiv(l,lambda i,v:{iname:i,vname:v})
    return(dl)


def vil_dict_after_vtrans(l,*other_args,trans_func=lambda r:r):
    desc = {}
    for i in range(0,l.__len__()):
        ele = l[i]
        nv = trans_func(ele,*other_args)
        if(nv in desc):
            desc[nv].append(i)
        else:
            desc[nv] = [i]
    return(desc)    





from setuptools import setup, find_packages


requirements = []

setup(
      name="xlist",
      version = "0.0.9", #@version@#
      description="handle,.in progressing..,APIs",
      author="ihgazni2",
      url="https://github.com/ihgazni2/xlist",
      author_email='', 
      license="MIT",
      long_description = "refer to .md files in https://github.com/ihgazni2/xlist",
      classifiers=[
          'Environment :: Console',
          'Environment :: Web Environment',
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'Programming Language :: Python',
          ],
      packages= find_packages(),
      entry_points={
          'console_scripts': [
              'xlist=xlist.bin:main'
          ]
      },
      package_data={
          'resources':['RESOURCES/*']
      },
      include_package_data=True,
      install_requires=requirements,
      py_modules=['xlist'], 
)


# python3 setup.py bdist --formats=tar
# python3 setup.py sdist













def uniform_index(index,lngth):
    if(index<0):
        rl = lngth+index
        if(rl<0):
            index = 0
        else:
            index = rl
    elif(index>=lngth):
        index = lngth
    else:
        index = index
    return(index)



def index_fst(ol,value):
    return(ol.index(value))


def index_fst_not(ol,value):
    lngth = ol.__len__()
    for i in range(0,lngth):
        if(value == ol[i]):
            pass
        else:
            return(i)
    return(None)


def index_lst(ol,value):
    length = ol.__len__()
    for i in range(length-1,-1,-1):
        if(value == ol[i]):
            return(i)
        else:
            pass
    return(None)



def index_lst_not(ol,value):
    length = ol.__len__()
    for i in range(length-1,-1,-1):
        if(value == ol[i]):
            pass
        else:
            return(i)
    return(None)


def index_which(ol,value,which):
    length = ol.__len__()
    seq = -1
    for i in range(0,length):
        if(value == ol[i]):
            seq = seq + 1
            if(seq == which):
                return(i)
            else:
                pass
        else:
            pass
    return(None)


def index_which_not(ol,value,which):
    length = ol.__len__()
    seq = -1
    for i in range(0,length):
        if(value == ol[i]):
            pass
        else:
            seq = seq + 1
            if(seq == which):
                return(i)
            else:
                pass
    return(None)


def indexes_all(ol,value):
    length = ol.__len__()
    indexes =[]
    for i in range(0,length):
        if(value == ol[i]):
            indexes.append(i)
        else:
            pass
    return(indexes)



def indexes_all_not(ol,value):
    length = ol.__len__()
    indexes =[]
    for i in range(0,length):
        if(value == ol[i]):
            pass
        else:
            indexes.append(i)
    return(indexes)

def indexes_some(ol,value,*seqs):
    seqs = list(seqs)
    length = ol.__len__()
    indexes =[]
    seq = -1
    for i in range(0,length):
        if(value == ol[i]):
            seq = seq + 1
            if(seq in seqs):
                indexes.append(i)
            else:
                pass
        else:
            pass
    return(indexes)



def indexes_some_not(ol,value,*seqs):
    seqs = list(seqs)
    length = ol.__len__()
    indexes =[]
    seq = -1
    for i in range(0,length):
        if(value == ol[i]):
            pass
        else:
            seq = seq + 1
            if(seq in seqs):
                indexes.append(i)
            else:
                pass
    return(indexes)



def indexes_fst_slice(ol,value):
    length = ol.__len__()
    begin = None
    slice = []
    for i in range(0,length):
        if(ol[i]==value):
            begin = i
            break
        else:
            pass
    if(begin == None):
        return(None)
    else:
        slice.append(begin)
        for i in range(begin+1,length):
            if(ol[i]==value):
                slice.append(i)
            else:
                break
    return(slice)



def indexes_fst_not_slice(ol,value):
    length = ol.__len__()
    begin = None
    slice = []
    for i in range(0,length):
        if(not(ol[i]==value)):
            begin = i
            break
        else:
            pass
    if(begin == None):
        return(None)
    else:
        slice.append(begin)
        for i in range(begin+1,length):
            if(not(ol[i]==value)):
                slice.append(i)
            else:
                break
    return(slice)


def indexes_lst_slice(ol,value):
    length = ol.__len__()
    end = None
    slice = []
    for i in range(length-1,-1,-1):
        if(ol[i]==value):
            end = i
            break
        else:
            pass
    if(end == None):
        return(None)
    else:
        slice.append(end)
        for i in range(end-1,-1,-1):
            if(ol[i]==value):
                slice.append(i)
            else:
                break
    slice.reverse()
    return(slice)



def indexes_lst_not_slice(ol,value):
    length = ol.__len__()
    end = None
    slice = []
    for i in range(length-1,-1,-1):
        if(not(ol[i]==value)):
            end = i
            break
        else:
            pass
    if(end == None):
        return(None)
    else:
        slice.append(end)
        for i in range(end-1,-1,-1):
            if(not(ol[i]==value)):
                slice.append(i)
            else:
                break
    slice.reverse()
    return(slice)


def indexes_which_slice(ol,value):
    length = ol.__len__()
    seq = -1
    cursor = 0
    begin = None
    slice = []
    while(cursor < length):
        cond1 = (ol[cursor] == value)
        cond2 = (begin == None)
        if(cond1 & cond2):
            begin = cursor
            slice.append(cursor)
            cursor = cursor + 1
        elif(cond1 & (not(cond2))):
            slice.append(cursor)
            cursor = cursor + 1
        elif((not(cond1)) & (not(cond2))):
            seq = seq + 1
            if(seq == which):
                return(slice)
            else:
                cursor = cursor + 1
                begin = None
                slice = []
        else:
            cursor = cursor + 1
    if(slice):
        seq = seq + 1
    else:
        pass
    if(seq == which):
        return(slice)
    else:
        return([])


def indexes_which_not_slice(ol,value):
    length = ol.__len__()
    seq = -1
    cursor = 0
    begin = None
    slice = []
    while(cursor < length):
        cond1 = not(ol[cursor] == value)
        cond2 = (begin == None)
        if(cond1 & cond2):
            begin = cursor
            slice.append(cursor)
            cursor = cursor + 1
        elif(cond1 & (not(cond2))):
            slice.append(cursor)
            cursor = cursor + 1
        elif((not(cond1)) & (not(cond2))):
            seq = seq + 1
            if(seq == which):
                return(slice)
            else:
                cursor = cursor + 1
                begin = None
                slice = []
        else:
            cursor = cursor + 1
    if(slice):
        seq = seq + 1
    else:
        pass
    if(seq == which):
        return(slice)
    else:
        return([])


def indexes_some_slices(ol,value,*seqs):
    seqs = list(seqs)
    rslt = []
    length = ol.__len__()
    seq = -1
    cursor = 0
    begin = None
    slice = []
    while(cursor < length):
        cond1 = (ol[cursor] == value)
        cond2 = (begin == None)
        if(cond1 & cond2):
            begin = cursor
            slice.append(cursor)
        elif(cond1 & (not(cond2))):
            slice.append(cursor)
        elif((not(cond1)) & (not(cond2))):
            seq = seq + 1
            if(seq in seqs):
                rslt.append(slice)
            else:
                pass
            begin = None
            slice = []
        else:
            pass
        cursor = cursor + 1
    if(slice):
        seq = seq + 1
        if(seq in seqs):
            rslt.append(slice)
        else:
            pass
    else:
        pass
    return(rslt)


def indexes_some_not_slices(ol,value,*seqs):
    seqs = list(seqs)
    rslt = []
    length = ol.__len__()
    seq = -1
    cursor = 0
    begin = None
    slice = []
    while(cursor < length):
        cond1 = not(ol[cursor] == value)
        cond2 = (begin == None)
        if(cond1 & cond2):
            begin = cursor
            slice.append(cursor)
        elif(cond1 & (not(cond2))):
            slice.append(cursor)
        elif((not(cond1)) & (not(cond2))):
            seq = seq + 1
            if(seq in seqs):
                rslt.append(slice)
            else:
                pass
            begin = None
            slice = []
        else:
            pass
        cursor = cursor + 1
    if(slice):
        seq = seq + 1
        if(seq in seqs):
            rslt.append(slice)
        else:
            pass
    else:
        pass
    return(rslt)


def indexes_all_slices(ol,value):
    rslt = []
    length = ol.__len__()
    cursor = 0
    begin = None
    slice = []
    while(cursor < length):
        cond1 = (ol[cursor] == value)
        cond2 = (begin == None)
        if(cond1 & cond2):
            begin = cursor
            slice.append(cursor)
        elif(cond1 & (not(cond2))):
            slice.append(cursor)
        elif((not(cond1)) & (not(cond2))):
            rslt.append(slice)
            begin = None
            slice = []
        else:
            pass
        cursor = cursor + 1
    if(slice):
        rslt.append(slice)
    else:
        pass
    return(rslt)


def indexes_all_not_slices(ol,value):
    rslt = []
    length = ol.__len__()
    cursor = 0
    begin = None
    slice = []
    while(cursor < length):
        cond1 = not(ol[cursor] == value)
        cond2 = (begin == None)
        if(cond1 & cond2):
            begin = cursor
            slice.append(cursor)
        elif(cond1 & (not(cond2))):
            slice.append(cursor)
        elif((not(cond1)) & (not(cond2))):
            rslt.append(slice)
            begin = None
            slice = []
        else:
            pass
        cursor = cursor + 1
    if(slice):
        rslt.append(slice)
    else:
        pass
    return(rslt)









import copy
from .slct  import some_keeping_order,some
from .index import uniform_index

def fcp(ol):
    return(ol[:])


def max_length(ol):
    lngths = list(map(lambda r:len(r),ol))
    lngth = max(lngths)
    return(lngth)


def entries(ol):
    rslt = []
    length = ol.__len__()
    for i in range(0,length):
        entry = [i,ol[i]]
        rslt.append(entry)
    return(rslt)


def includes(ol,value):
    return((value in ol))


def to_str(ol):
    return(ol.__str__())


def to_src(ol):
    return(ol.__repr__())


def every(ol,test_func,*args,**kwargs):
    rslt = True
    length = ol.__len__()
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(cond):
            pass
        else:
            return(False)
    return(rslt)


def any(ol,test_func,*args,**kwargs):
    rslt = False
    length = ol.__len__()
    for i in range(0,length):
        cond = test_func(ol[i],*args)
        if(cond):
            return(True)
        else:
            pass
    return(rslt)


def uniqualize(l,**kwargs):
    if('mode' in kwargs):
        mode = kwargs['mode']
    else:
        mode = 'deepcopy'
    pt = copy.deepcopy(l)
    seqs =[]
    freq = {}
    for i in range(0,pt.__len__()):
        v = pt[i]
        if(v in freq):
            freq[v] = freq[v] + 1
        else:
            freq[v] = 0
            seqs.append(i)
    #####下面是影响速度的关键,append特别耗时
    #npt = some_keeping_order(pt,*seqs)
    npt = some(pt,*seqs)
    ########################
    pt = npt
    if(mode == 'deepcopy'):
        return(npt)
    else:
        l.clear()
        l.extend(npt)
        return(l)

def combinations(arr,*args):
    args = list(args)
    lngth = len(args)
    if(lngth == 0):
        start = 1
        end = len(arr) + 1
    elif(lngth == 1):
        start = uniform_index(args[0],lngth)
        end = len(arr) + 1
    else:
        start = uniform_index(args[0],lngth)
        end = uniform_index(args[1],lngth)
    rslt = []
    for i in range(start,end):
        tmp = list(itertools.combinations(arr,i))
        rslt.extend(tmp)
    return(rslt)




from .cmmn import deepcopy_wrapper
from .cmmn import inplace_wrapper
from .cmmn import identity
import copy
import inspect
from .slct import odds,evens


def init(lngth,*args,**kwargs):
    deepcopy = kwargs['deepcopy'] if('deepcopy' in kwargs) else True
    dflt_ele = args[0] if(len(args)>0) else None
    rslt = []
    for i in range(0,lngth):
        dflt_ele = copy.deepcopy(dflt_ele) if(deepcopy) else dflt_ele
        rslt.append(dflt_ele)
    return(rslt)

def from_range(si,ei,step=1):
    return(list(range(si,ei,step)))


@inplace_wrapper
def fill(ol,value,si=None,ei=None,**kwargs):
    lngth = len(ol)
    si = 0 if(si==None) else si
    ei = lngth if(ei==None) else ei
    for i in range(si,ei):
        ol[i] = value
    return(ol)


@deepcopy_wrapper
def lfrom(obj,*args,**kwargs):
    func = args[0] if(len(args)>0) else identity
    spec = inspect.getargspec(func)
    params_lngth = len(spec.args)
    has_args = (spec.varargs!=None)
    has_kw = (spec.keywords!=None)
    other_args = args[1:] 
    ol = list(obj)
    nl = []
    for i in range(len(ol)):
        if(params_lngth==1 and has_args and has_kw):
            ele = func(ol[i],*other_args,**kwargs)
        elif(params_lngth==1 and has_args and not(has_kw)):
            ele = func(ol[i],*other_args)
        elif(params_lngth==1 and not(has_args) and has_kw):
            ele = func(ol[i],**kwargs)
        elif(params_lngth==1 and not(has_args) and not(has_kw)):
            ele = func(ol[i])
        elif(params_lngth==2 and has_args and has_kw):
            ele = func(ol[i],i,*other_args,**kwargs)
        elif(params_lngth==2 and has_args and not(has_kw)):
            ele = func(ol[i],i,*other_args)
        elif(params_lngth==2 and not(has_args) and has_kw):
            ele = func(ol[i],i,**kwargs)
        elif(params_lngth==2 and not(has_args) and not(has_kw)):
            ele = func(ol[i],i)
        else:
            raise(TypeError("type error!")) 
        nl.append(ele)
    return(nl)



def of(*eles):
    return(list(eles))




def from_one_list(ol,*args,**kwargs):
    nl = ol
    args = list(args)
    if(isinstance(args,list)):
        indexes = args[0]
        values = args[1]
    else:
        indexes = slct.odds(ol)
        values = slct.evens(ol)
    lngth = indexes.__len__()
    for i in range(lngth):
        index = indexes[i]
        value = values[i]
        nl[index] = value
    return(nl)
